import pathlib

PACKAGE_DIRECTORY = str(pathlib.Path(__file__).parent.resolve())
TEST_DATA_FOLDER = str(PACKAGE_DIRECTORY) + "/data"
PACKAGE_SECRETS_FILE = str(PACKAGE_DIRECTORY) + "/secrets.env"


class Show:
    HEADER = '\033[95m'
    OK_BLUE = '\033[94m'
    OK_CYAN = '\033[96m'
    OK_GREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    END = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'

    @classmethod
    def warning(cls, text):
        print(f'{cls.WARNING} {text}{cls.END}')

    @classmethod
    def info(cls, text):
        print(f'{cls.OK_BLUE} {text}{cls.END}')

    @classmethod
    def success(cls, text):
        print(f'{cls.BOLD}{cls.OK_GREEN} {text}{cls.END}')

    @classmethod
    def error(cls, text):
        print(f'{cls.FAIL} {text}{cls.END}')


import os
import argparse

from ._config import PACKAGE_SECRETS_FILE, Show
from ._helpers import TEST_CLASSES, TEST_CLASSES_LIST


def handle_setup(args):
    Show.warning("Dragon is setting up your username and password")
    with open(PACKAGE_SECRETS_FILE, "w") as secret_file:
        secret_file.write(args.username + "\n")
        secret_file.write(args.password + "\n")
        secret_file.write(args.server + "\n")
    Show.success("Set up has completed successfully. Run dragon -h to see more on how to use")


def run_tests(test):
    test_class = TEST_CLASSES.get(test)
    if test_class:
        Show.warning("Loading done, now running tests, please wait ...... ")
        test_class().submit_test_form()
        exit()


def handle_program(args, parser):
    if args.program == "help":
        Show.warning(parser.format_help())
    elif args.program == "update":
        Show.info("Dragon is updating it's resources please wait")
        Show.error('This function has not been implemented at the moment')
    elif args.program == "list":
        Show.error('Below are the tests available at the moment')
        for test in TEST_CLASSES_LIST:
            Show.info(f'{test}')
        Show.success('Please run dragon test {name_of_class} example: dragon test sample_test')
    else:
        Show.warning(f"Dragon is loading test for {args.program}")
        run_tests(args.program)


def parse_commandline():
    parser = argparse.ArgumentParser(prog='dragon')
    subparsers = parser.add_subparsers(help='sub-command help', dest="command",
                                       description="The main category command to be run", title="command",
                                       required=True)

    parser_test = subparsers.add_parser('test', help='Run the program commands', )
    choices = ['list', 'update', 'help'] + TEST_CLASSES_LIST
    parser_test.add_argument('program', choices=choices,
                             help='The command you would like to run. Run ')

    parser_setup = subparsers.add_parser('setup', help='Run the initial setup for the program')
    parser_setup.add_argument("-u", dest="username", required=True)
    parser_setup.add_argument("-p", dest="password", required=True)
    parser_setup.add_argument("-s", dest="server", required=True)
    return parser


def main():
    Show.warning("Dragon is loading your commands!!")
    parser = parse_commandline()
    args = parser.parse_args()
    if args.command == 'setup':
        handle_setup(args)
    else:
        handle_program(args, parser)


if __name__ == '__main__':
    main()



from ._config import TEST_DATA_FOLDER
from .base import BaseTest
from selenium.webdriver.common.by import By


class CramDeliveryRequest(BaseTest):
    def setup(self):
        self.log_in_and_visit_page('jira/wgs_delivery_request/')
        self._fill_in_form_with_default_values()

    def _fill_in_form_with_default_values(self):
        self.fill_in_form_field("workspace_name", "some_workspace")
        self.fill_in_form_field("cohort_name", "test_cohort_name")
        self.fill_in_form_field("project_name", "test_project_name")
        self.fill_in_form_field("internal_pi", "test_internal_pi")
        self.fill_in_form_field("discoverability", "test_internal_pi")
        self.fill_in_form_field("race_ethnicity", "Race")
        self.fill_in_form_field("age_specified", "23")
        self.fill_in_form_field("disease_description", "EPI")
        self.fill_in_form_field("primary_disease", "Cancer")
        self.select_radio_button("new_workspace", "no")
        self.select_radio_button("override_delivery", "yes")
        self.select_radio_button("data_type", "WGS")
        self.select_radio_button("deliver_metadata", "yes")
        self.select_radio_button("fee_for_service", "no")
        self.select_radio_button("clinical_data_delivery", "yes")
        self.select_radio_button("sample_threshold", "AUTOMATIC")
        self.select_radio_button("self_service_option", "manual_submission")
        self.select_radio_button("move_gvcfs", "no")
        self.upload_file_from_path('file_field',
                                   f'{TEST_DATA_FOLDER}/test_cram_v2_clinical_delivery_metadata.txt')

    def submit_test_form(self):
        self.setup()
        self.selenium.find_element(By.NAME, 'file_field').submit()
        self.assert_form_submitted_successfully()
        link = self.selenium.find_element(By.PARTIAL_LINK_TEXT, 'https://broadinstitute.atlassian.net/browse/POTE').text
        print(f"Issue has been created at {link}")



import time
import unittest
from selenium import webdriver
import chromedriver_autoinstaller
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By

from ._config import PACKAGE_SECRETS_FILE, Show

chromedriver_autoinstaller.install()


class BaseTest(unittest.TestCase):
    def __init__(self, login_page="dopsadmin/login/", email="test"):
        super().__init__()
        self.email = email + '@broadinstitute.org'
        self.selenium = webdriver.Chrome(options=self.chrome_options)
        self.login_page = login_page
        self.username, self.password, self.server_url = self.set_credentials()

    @staticmethod
    def set_credentials():
        try:
            with open(PACKAGE_SECRETS_FILE, "r") as secrets_file:
                username = secrets_file.readline().rstrip()
                password = secrets_file.readline().rstrip()
                server_url = secrets_file.readline().rstrip()
            return username, password, server_url
        except FileNotFoundError:
            Show.error("It seems like you have not run setup")
            Show.info(
                "run# dragon setup -u username -p password -s https://the_server_url_without_trailing_forward_slash")
            exit()

    @property
    def chrome_options(self):
        chrome_options = Options()
        chrome_options.add_argument("--headless")
        chrome_options.add_argument("--disable-gpu")
        return chrome_options

    def visit_page(self, path):
        self.selenium.get('/'.join([self.server_url, path]))

    def log_in(self):
        self.visit_page(self.login_page)
        self.fill_in_form_field('username', self.username)
        self.fill_in_form_field('password', self.password)
        self.find_element(By.NAME, 'username').submit()
        try:
            Show.warning(f"Loading page at {self.server_url} please wait")
            time.sleep(5)
            self.assertTrue(self.selenium.find_element(By.ID, 'user-tools'))
        except NoSuchElementException:
            print(self.selenium.page_source)
            Show.error("You have used the wrong credentials.")
            Show.info("Confirm your credentials and run dragon setup -u username -p password -s https://server_url")
            exit()

    def log_in_and_visit_page(self, path):
        self.log_in()
        self.visit_page(path)

    def find_element(self, by, condition_value):
        try:
            target = self.selenium.find_element(by, condition_value)
            return target
        except NoSuchElementException:
            print(self.selenium.page_source)
            Show.error(f"Not able to locate element {condition_value} by {str(by)}")
            Show.info(f"Confirm your credentials and run dragon setup -u username -p password -s https://server_url")
            exit()

    def fill_in_form_field(self, field_name, value):
        target_field = self.find_element(By.NAME, field_name)
        target_field.send_keys(value)

    def select_radio_button(self, field_name, value):
        target_button = self.find_element(By.CSS_SELECTOR,
                                          f"input[type='radio'][name='{field_name}'][value='{value}']")
        target_button.click()

    def select_select_option(self, field_name, value):
        target_button = self.find_element(By.CSS_SELECTOR,
                                          f"select[name='{field_name}']>option[value='{value}']")
        target_button.click()

    def select_checkbox(self, field_name):
        target_button = self.find_element(By.CSS_SELECTOR, f"input[type='checkbox'][name='{field_name}']")
        target_button.click()

    def upload_file_from_path(self, field_name, file_path):
        target_field = self.find_element(By.NAME, field_name)
        target_field.send_keys(file_path)

    def assert_form_submitted_successfully(self):
        jira_link = self.find_element(By.ID, 'delivery-success-header')
        self.assertTrue(jira_link)
        main_content = self.find_element(By.ID, 'main-content')
        self.assertIn('issue was created for you', main_content.text)



from .move_data import CramDeliveryRequest
from .sample_test import SampleTest

TEST_CLASSES = {
    "sample_test": SampleTest,
    "clinical": CramDeliveryRequest,
    "anvil_arrays": CramDeliveryRequest,
    "anvil_crams": CramDeliveryRequest
}

TEST_CLASSES_LIST = list(TEST_CLASSES.keys())



from setuptools import setup, find_packages
import pathlib

PACKAGE_DIRECTORY = pathlib.Path(__file__).parent.resolve()

long_description = (PACKAGE_DIRECTORY / 'README.md').read_text(encoding='utf-8')

setup(
    name='dragons_fire',
    description='A test suite to provide simple testing of workflows',
    version='0.1.8',
    url='https://github.com/kilonzi/gragons',
    author='John Kitonyo',
    author_email='jkitonyo@broadinstitute.org',
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3 :: Only',
    ],
    keywords='dragon, tests, tickets',
    packages=find_packages(),
    python_requires='>=3.6, <4',
    install_requires=['selenium==3.141.0', 'chromedriver-autoinstaller'],
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'dragon=dragon:main',
        ],
    },
)
